@@ -3,7 +3,6 @@ import json
 import logging
 import os
 import sys
-import yaml
 
 from json_schema_validator.errors import ValidationError
 from lava.tool.command import Command
@@ -12,10 +11,10 @@ from lava.dispatcher.node import NodeDispatcher
 import lava_dispatcher.config
 from lava_dispatcher.config import get_config, get_device_config, list_devices
 from lava_dispatcher.job import LavaTestJob, validate_job_data
-from lava_dispatcher.pipeline.parser import JobParser
-from lava_dispatcher.context import LavaContext
-from lava_dispatcher.pipeline.action import Device
+from lava_dispatcher.pipeline.action import JobError
+from lava_dispatcher.pipeline.log import YAMLLogger
 from lava_dispatcher.pipeline.device import NewDevice
+from lava_dispatcher.pipeline.parser import JobParser
 
 
 class SetUserConfigDirAction(argparse.Action):
@@ -73,30 +72,37 @@ def run_legacy_job(job_data, oob_file, config, output_dir, validate):
 
 
 def get_pipeline_runner(job):
+    """
+    Accepts a Pipeline job created by the pipeline parser which will have
+    parameters populated by the parser.
+    """
     # additional arguments are now inside the context
 
+    # FIXME: drop outdated arguments, job_data, config and output_dir
     def run_pipeline_job(job_data, oob_file, config, output_dir, validate_only):
-        # pipeline actions will add their own handlers.
-        yaml_log = logging.getLogger("YAML")
-        yaml_log.setLevel(logging.DEBUG)
-        std_log = logging.getLogger("ASCII")
-
-        stdhandler = logging.StreamHandler(oob_file)
-        stdhandler.setLevel(logging.INFO)
-        formatter = logging.Formatter('"%(asctime)s"\n%(message)s')
-        stdhandler.setFormatter(formatter)
-        std_log.addHandler(stdhandler)
-
+        # always validate every pipeline before attempting to run.
+        exitcode = 0
         try:
             job.validate(simulate=validate_only)
             if not validate_only:
-                job.run()
-        except lava_dispatcher.pipeline.JobError as e:
-            print(e)
+                exitcode = job.run()
+        except JobError as exc:
+            print exc
+            logging.debug("%s" % exc)
             sys.exit(2)
+        if exitcode:
+            sys.exit(exitcode)
+        # FIXME: should we call the cleanup function in the finally block?
     return run_pipeline_job
 
 
+def is_pipeline_job(filename):
+    # FIXME: use the schema once it is available
+    if filename.lower().endswith('.yaml') or filename.lower().endswith('.yml'):
+        return True
+    return False
+
+
 class dispatch(DispatcherCommand):
     """
     Run test scenarios on virtual and physical hardware
@@ -117,10 +123,14 @@ class dispatch(DispatcherCommand):
         parser.add_argument(
             "--validate", action='store_true',
             help="Just validate the job file, do not execute any steps.")
+        # TODO: this is required for the new dispatcher
         parser.add_argument(
             "--job-id", action='store', default=None,
             help=("Set the scheduler job identifier. "
                   "This alters process name for easier debugging"))
+        parser.add_argument(
+            "--socket-addr", default=None,
+            help="Address of the ZMQ socket used to send the logs to the master")
         parser.add_argument(
             "job_file",
             metavar="JOB",
@@ -132,6 +142,13 @@ class dispatch(DispatcherCommand):
         )
 
     def invoke(self):
+        """
+        Entry point for lava dispatch, after the arguments have been parsed.
+        """
+
+        if os.geteuid() != 0:
+            logging.error("You need to be root to run lava-dispatch.")
+            exit(1)
 
         if self.args.oob_fd:
             oob_file = os.fdopen(self.args.oob_fd, 'w')
@@ -144,36 +161,49 @@ class dispatch(DispatcherCommand):
         # done.
         del logging.root.handlers[:]
         del logging.root.filters[:]
-        FORMAT = '<LAVA_DISPATCHER>%(asctime)s %(levelname)s: %(message)s'
-        DATEFMT = '%Y-%m-%d %I:%M:%S %p'
-        logging.basicConfig(format=FORMAT, datefmt=DATEFMT)
-        try:
-            self.config = get_config()
-        except CommandError as e:
-            if self.args.output_dir:
-                reporter = os.path.join(self.args.output_dir, "output.txt")
-                with open(reporter, 'a') as f:
-                    f.write("Configuration error: %s\n" % e)
-            else:
-                print(e)
-            exit(1)
-        logging.root.setLevel(self.config.logging_level)
+        if is_pipeline_job(self.args.job_file):
+            # Branch point for the pipeline code - currently reliant on a simple filename
+            # extension match.
+            self.config = None  # external config is loaded on-demand by the pipeline
+            # pipeline *always* logs at debug level, so do not set from config.
+
+            # Pipeline always log as YAML so change the base logger.
+            # Every calls to logging.getLogger will now return a YAMLLogger.
+            logging.setLoggerClass(YAMLLogger)
+        else:
+            FORMAT = '<LAVA_DISPATCHER>%(asctime)s %(levelname)s: %(message)s'
+            DATEFMT = '%Y-%m-%d %I:%M:%S %p'
+            logging.basicConfig(format=FORMAT, datefmt=DATEFMT)
+            try:
+                self.config = get_config()
+            except CommandError as e:
+                if self.args.output_dir:
+                    reporter = os.path.join(self.args.output_dir, "output.txt")
+                    with open(reporter, 'a') as f:
+                        f.write("Configuration error: %s\n" % e)
+                else:
+                    print e
+                exit(1)
+            # Set the logging level
+            logging.root.setLevel(self.config.logging_level)
 
         # Set process id if job-id was passed to dispatcher
         if self.args.job_id:
             try:
-                from setproctitle import getproctitle, setproctitle
+                from setproctitle import setproctitle
             except ImportError:
                 logging.warning(
                     ("Unable to set import 'setproctitle', "
                      "process name cannot be changed"))
             else:
-                setproctitle("%s [job: %s]" % (
-                    getproctitle(), self.args.job_id))
+                setproctitle("lava-dispatch [job: %s]" % self.args.job_id)
 
         # Load the job file
         job_runner, job_data = self.parse_job_file(self.args.job_file, oob_file)
 
+        if self.args.output_dir and not os.path.isdir(self.args.output_dir):
+            os.makedirs(self.args.output_dir)
+
         # detect multinode and start a NodeDispatcher to work with the LAVA Coordinator.
         if not self.args.validate:
             if 'target_group' in job_data:
@@ -188,26 +218,32 @@ class dispatch(DispatcherCommand):
                 exit(1)
         else:
             job_data['target'] = self.args.target
-        if self.args.output_dir and not os.path.isdir(self.args.output_dir):
-            os.makedirs(self.args.output_dir)
 
         job_runner(job_data, oob_file, self.config, self.args.output_dir, self.args.validate)
 
     def parse_job_file(self, filename, oob_file):
-        if filename.lower().endswith('.yaml') or filename.lower().endswith('.yml'):
+        """
+        Uses the parsed device_config instead of the old Device class
+        so it can fail before the Pipeline is made.
+        Avoids loading all configuration for all supported devices for every job.
+        """
+        if is_pipeline_job(filename):
+            # Prepare the pipeline from the file using the parser.
 
-            device = NewDevice(self.args.target)
-            # FIXME: paths not standardised, so can't work from the command line yet.
-            if not device.parameters:
-                device = Device(self.args.target)
+            device = NewDevice(self.args.target)  # DeviceParser
             parser = JobParser()
-            # FIXME: use the parsed device_config instead of the old Device class so it can fail before the Pipeline is made.
-            job = parser.parse(open(filename), device, output_dir=self.args.output_dir)
+            job = None
+            try:
+                with open(filename) as f_in:
+                    job = parser.parse(f_in, device, self.args.job_id,
+                                       socket_addr=self.args.socket_addr,
+                                       output_dir=self.args.output_dir)
+
+            except JobError as exc:
+                logging.error("Invalid job submission: %s" % exc)
+                exit(1)
+            # FIXME: NewDevice schema needs a validation parser
             # device.check_config(job)
-            if 'target_group' in job.parameters:
-                raise RuntimeError("Pipeline dispatcher does not yet support MultiNode")
-            # TODO: job.parameters isn't really needed in the call to the context, remove later.
-            job.context = LavaContext(self.args.target, self.config, oob_file, job.parameters, self.args.output_dir)
             return get_pipeline_runner(job), job.parameters
 
         # everything else is assumed to be JSON
