@@ -72,17 +72,11 @@
 #
 # /lava-$(DEVICE_HOSTNAME)/
 #    results/
-#       hwcontext/                 Each test_run in the bundle has the same
-#                                  hw & sw context info attached to it.
-#          cpuinfo.txt             Hardware info.
-#          meminfo.txt             Ditto.
-#       swcontext/
-#          build.txt               Software info.
-#          pkgs.txt                Ditto
 #       ${IDX}_${TEST_ID}-${TIMESTAMP}/
 #          testdef.yml
 #          testdef_metadata
 #          stdout.log
+#          install_return_code             The exit code of install.sh.
 #          return_code             The exit code of run.sh.
 #          analyzer_assigned_uuid
 #          attachments/
@@ -107,13 +101,22 @@
 #                attachments/      Contains attachments for test results.
 #                   ${FILENAME}           The attached data.
 #                   ${FILENAME}.mimetype  The mime type of the attachment.
+#           hwcontext/                 Each test_run in the bundle has the same
+#                                  hw & sw context info attached to it.
+#               cpuinfo.txt             Hardware info.
+#               meminfo.txt             Ditto.
+#           swcontext/
+#               build.txt               Software info.
+#               pkgs.txt                Ditto
 #
 # After the test run has completed, the /lava-$(DEVICE_HOSTNAME)/results
 # directory is pulled over to the host and turned into a bundle for submission
 # to the dashboard.
 
+import re
 from datetime import datetime
 from glob import glob
+import ast
 import base64
 import logging
 import os
@@ -126,6 +129,7 @@ import subprocess
 import tarfile
 import tempfile
 import time
+import sys
 from uuid import uuid4
 
 import yaml
@@ -161,24 +165,50 @@ XMOD = stat.S_IRWXU | stat.S_IXGRP | stat.S_IRGRP | stat.S_IXOTH | stat.S_IROTH
 
 INVALID_CHARS = " $&()\"'<>/\\|;`"
 
+repeat_cnt = 0
+
+
+# Make the open function accept encodings in python < 3.x
+if sys.version_info[0] < 3:
+    import codecs
+    open = codecs.open
+
+
+def _validate_invalid_chars(parameter):
+    i = []
+    for e in INVALID_CHARS:
+        i.extend(utils.indices(parameter, e))
+    if i:
+        msg = "Parameter '%s' contains invalid symbol(s) at position(s): %s" % \
+              (parameter, ", ".join(map(str, i)))
+        raise GeneralError(msg)
+
 
 def _get_lava_proxy(context):
     return {'http_proxy': context.config.lava_proxy,
-            'https_proxy': context.config.lava_proxy}
+            'https_proxy': context.config.lava_proxy,
+            'no_proxy': context.config.lava_no_proxy}
 
 
 def _get_testdef_git_repo(testdef_repo, tmpdir, revision, proxy_env):
     cwd = os.getcwd()
     gitdir = os.path.join(tmpdir, 'gittestrepo')
     try:
-        subprocess.check_call(['git', 'clone', testdef_repo, gitdir],
-                              env=proxy_env)
+        subprocess.check_output(['git', 'clone', testdef_repo, gitdir],
+                                env=proxy_env, stderr=subprocess.STDOUT)
         if revision:
             os.chdir(gitdir)
-            subprocess.check_call(['git', 'checkout', revision])
+            subprocess.check_output(['git', 'checkout', revision],
+                                    stderr=subprocess.STDOUT)
         return gitdir
-    except Exception as e:
-        logging.error("Unable to get test definition from git (%s)" % (testdef_repo))
+    except subprocess.CalledProcessError as e:
+        logging.error("Test Shell Error: Unable to get test definition from git (%s)", (testdef_repo))
+        for line in e.output.split('\n'):
+            if line:
+                logging.debug("  | %s", line)
+        raise RuntimeError("Unable to get test definition from git (%s)" % (testdef_repo))
+    except Exception:
+        logging.error("Test Shell Error: Unable to get test definition from git (%s)", (testdef_repo))
         raise RuntimeError("Unable to get test definition from git (%s)" % (testdef_repo))
     finally:
         os.chdir(cwd)
@@ -193,13 +223,19 @@ def _get_testdef_bzr_repo(testdef_repo, tmpdir, revision, proxy_env):
             revision = '-1'
 
         proxy_env.update({'BZR_HOME': '/dev/null', 'BZR_LOG': '/dev/null'})
-        subprocess.check_call(
-            ['bzr', 'branch', '-r', revision, testdef_repo, bzrdir],
-            env=proxy_env)
+        subprocess.check_output(['bzr', 'branch', '-r', revision, testdef_repo,
+                                 bzrdir], env=proxy_env)
         return bzrdir
+    except subprocess.CalledProcessError as e:
+        logging.error("Test Error: Unable to get test definition from bzr (%s)", (testdef_repo))
+        for line in e.output.split('\n'):
+            if line:
+                logging.debug("  | %s", line)
+        raise RuntimeError("Unable to get test definition from bzr (%s)", (testdef_repo))
+
     except Exception as e:
-        logging.error("Unable to get test definition from bzr (%s)" % (testdef_repo))
-        raise RuntimeError("Unable to get test definition from bzr (%s)" % (testdef_repo))
+        logging.error("Test Error: Unable to get test definition from bzr (%s)", (testdef_repo))
+        raise RuntimeError("Unable to get test definition from bzr (%s)", (testdef_repo))
 
 
 def _get_testdef_tar_repo(testdef_repo, tmpdir):
@@ -225,7 +261,7 @@ def _get_testdef_tar_repo(testdef_repo, tmpdir):
         with tarfile.open(temp_tar) as tar:
             tar.extractall(path=tardir)
     except (OSError, tarfile.TarError) as ex:
-        logging.error("Error extracting the tar archive.\n" + str(ex))
+        logging.error("Test Shell Error: Error extracting the tar archive.\n" + str(ex))
     finally:
         # Remove the temporary created tar file after it has been extracted.
         if os.path.isfile(temp_tar):
@@ -245,7 +281,7 @@ def _get_testdef_url_repo(testdef_repo, context, tmpdir):
         testdef_file = download_image(testdef_repo, context, urldir)
 
     except Exception as e:
-        logging.error('Unable to get test definition from url\n' + str(e))
+        logging.error('Test Shell Error: Unable to get test definition from url\n' + str(e))
         return None
     finally:
         logging.info("Downloaded test definition file to %s." % urldir)
@@ -254,7 +290,12 @@ def _get_testdef_url_repo(testdef_repo, context, tmpdir):
 
 
 def _get_testdef_info(testdef):
+
+    # before populating metadata, validate the testdef name.
+    _validate_invalid_chars(testdef['metadata']['name'])
+
     metadata = {'os': '', 'devices': '', 'environment': ''}
+    metadata['name'] = testdef['metadata']['name']
     metadata['description'] = testdef['metadata'].get('description')
     metadata['format'] = testdef['metadata'].get('format')
     version = testdef['metadata'].get('version')
@@ -302,25 +343,12 @@ class TestDefinitionLoader(object):
                 self.load_from_repo(testcase)
 
     def load_from_url(self, url):
-        i = []
         tmpdir = utils.mkdtemp(self.tmpbase)
         testdef_file = download_image(url, self.context, tmpdir)
         with open(testdef_file, 'r') as f:
             logging.debug('loading test definition ...')
             testdef = yaml.safe_load(f)
 
-        for e in INVALID_CHARS:
-            i.extend(indices(testdef["metadata"]["name"], e))
-        if i:
-            msg = "Test name contains invalid symbol(s) at position(s): %s" % ", ".join(map(str, i))
-            raise GeneralError(msg)
-
-        try:
-            testdef["metadata"]["name"].encode()
-        except UnicodeEncodeError as e:
-            msg = "Test name contains non-ascii symbols: %s" % e
-            raise GeneralError(msg)
-
         if 'test-case-deps' in testdef:
             self._get_dependent_test_cases(testdef)
 
@@ -390,13 +418,14 @@ class TestDefinitionLoader(object):
                     logging.debug('loading test definition ...')
                     testdef = yaml.safe_load(f)
             except IOError as e:
-                msg = "Unable to load test definition '%s/%s': %s" % (os.path.basename(repo), test, e)
-                raise CriticalError(msg)
+                msg = "Test Shell Error: Unable to load test definition '%s/%s': %s" % (os.path.basename(repo), test, e)
+                logging.error(msg)
+                raise RuntimeError(msg)
 
             if 'test-case-deps' in testdef:
                 self._get_dependent_test_cases(testdef)
 
-            # for test paramters
+            # for test parameters
             if 'params' in testdef:
                 logging.debug('Get default parameters : %s' % testdef['params'])
                 info['default_params'] = str(testdef['params'])
@@ -408,11 +437,6 @@ class TestDefinitionLoader(object):
                 RepoTestDefinition(self.context, idx, testdef, repo, info))
 
 
-def indices(string, char):
-
-    return [i for i, c in enumerate(string) if c == char]
-
-
 def _bzr_info(url, bzrdir, name):
     cwd = os.getcwd()
     try:
@@ -454,14 +478,15 @@ class URLTestDefinition(object):
         self.testdef = testdef
         self.testdef_metadata = testdef_metadata
         self.idx = idx
-        self.test_id = self.testdef['metadata']['name']
-        self.dirname = '%s_%s' % (idx, self.test_id)
         self.uuid = str(uuid4())
+        self.test_id = self.testdef_metadata['name']
+        self.dirname = '%s_%s' % (idx, self.uuid)
         self._sw_sources = []
         self.handler = None
         self.__pattern__ = None
         self.__fixupdict__ = None
         self.skip_install = None
+        self.all_params = {}
 
     def load_signal_handler(self):
         hook_data = self.testdef.get('handler')
@@ -474,7 +499,7 @@ class URLTestDefinition(object):
                 pkg_resources.iter_entry_points(
                     'lava.signal_handlers', handler_name))
             if len(handler_eps) == 0:
-                logging.error("No handler named %s found", handler_name)
+                logging.error("Test Shell Error: No handler named %s found", handler_name)
                 return
             elif len(handler_eps) > 1:
                 logging.warning(
@@ -510,16 +535,61 @@ class URLTestDefinition(object):
                 self._sw_sources.append(_bzr_info(repo, name, name))
 
             for repo in self.testdef['install'].get('git-repos', []):
-                logging.info("git clone %s" % repo)
-                subprocess.check_output(['git', 'clone', repo],
-                                        env=_get_lava_proxy(self.context),
-                                        stderr=subprocess.STDOUT)
-                name = os.path.splitext(os.path.basename(repo))[0]
-                self._sw_sources.append(_git_info(repo, name, name))
+                if isinstance(repo, str):
+                    logging.info("git clone %s" % repo)
+                    subprocess.check_output(['git', 'clone', repo],
+                                            env=_get_lava_proxy(self.context),
+                                            stderr=subprocess.STDOUT)
+                    name = gitdir = os.path.splitext(os.path.basename(repo))[0]
+                    self._sw_sources.append(_git_info(repo, gitdir, name))
+                if isinstance(repo, dict):
+                    cmd = ['git', 'clone']
+                    # Check if this repository should be skipped.
+                    skip_by_default = repo.get('skip_by_default', False)
+                    if skip_by_default in self.all_params:
+                        if ast.literal_eval(self.all_params[skip_by_default]):
+                            continue
+
+                    url = repo.get('url', None)
+                    branch = repo.get('branch', None)
+                    if branch in self.all_params:
+                        branch = self.all_params[branch]
+                    destination = repo.get('destination', None)
+                    if destination in self.all_params:
+                        destination = self.all_params[destination]
+
+                    # Form the command list
+                    cmd = ['git', 'clone']
+                    if branch:
+                        cmd.append('-b')
+                        cmd.append(branch)
+                    if url:
+                        cmd.append(url)
+                    if destination:
+                        cmd.append(destination)
+
+                    logging.info("git clone %s" % url)
+                    subprocess.check_output(cmd,
+                                            env=_get_lava_proxy(self.context),
+                                            stderr=subprocess.STDOUT)
+                    name = os.path.splitext(os.path.basename(url))[0]
+                    gitdir = destination if destination else name
+                    self._sw_sources.append(_git_info(url, gitdir, name))
         finally:
             os.chdir(cwd)
 
+    def _fetch_all_parameters(self):
+        # default parameters that was defined in yaml
+        if 'params' in self.testdef:
+            self.all_params.update(self.testdef['params'])
+
+        # parameters that was set in json
+        if self._sw_sources and 'test_params' in self._sw_sources[0] and self._sw_sources[0]['test_params'] != '':
+            _test_params_temp = eval(self._sw_sources[0]['test_params'])
+            self.all_params.update(_test_params_temp)
+
     def _inject_testdef_parameters(self, fout):
+        global repeat_cnt
         # inject default parameters that was defined in yaml first
         fout.write('###default parameters from yaml###\n')
         if 'params' in self.testdef:
@@ -533,9 +603,21 @@ class URLTestDefinition(object):
             for param_name, param_value in _test_params_temp.items():
                 fout.write('%s=\'%s\'\n' % (param_name, param_value))
         fout.write('######\n')
+        # inject other parameters
+        target_type = self.context.client.target_device.deployment_data.get(
+            'distro')
+        fout.write('###other parameters###\n')
+        fout.write('%s=\'%s\'\n' % ('LAVA_SERVER_IP',
+                                    self.context.config.lava_server_ip))
+        fout.write('%s=\'%s\'\n' % ('TARGET_TYPE', target_type))
+        if repeat_cnt > 0:
+            fout.write('%s=\'%s\'\n' % ('REPEAT_COUNT', repeat_cnt))
+            fout.write('###save REPEAT_COUNT as named attribute###\n')
+            fout.write('echo %s > $LAVA_RESULT_DIR/attributes/repeat_count\n' % repeat_cnt)
+        fout.write('######\n')
 
     def _create_target_install(self, hostdir, targetdir):
-        with open('%s/install.sh' % hostdir, 'w') as f:
+        with open('%s/install.sh' % hostdir, 'w', encoding='utf-8') as f:
             self._inject_testdef_parameters(f)
             f.write('set -ex\n')
             f.write('cd %s\n' % targetdir)
@@ -586,13 +668,14 @@ class URLTestDefinition(object):
             boots.
         """
         utils.ensure_directory(hostdir)
+        self._fetch_all_parameters()
         with open('%s/testdef.yaml' % hostdir, 'w') as f:
-            f.write(yaml.dump(self.testdef))
+            f.write(yaml.dump(self.testdef, encoding='utf-8', allow_unicode=True))
 
-        with open('%s/uuid' % hostdir, 'w') as f:
+        with open('%s/uuid' % hostdir, 'w', encoding='utf-8') as f:
             f.write(self.uuid)
 
-        with open('%s/testdef_metadata' % hostdir, 'w') as f:
+        with open('%s/testdef_metadata' % hostdir, 'w', encoding='utf-8') as f:
             f.write(yaml.safe_dump(self.testdef_metadata))
 
         if self.skip_install != "all":
@@ -601,7 +684,7 @@ class URLTestDefinition(object):
                     self._create_repos(hostdir)
                 self._create_target_install(hostdir, targetdir)
 
-        with open('%s/run.sh' % hostdir, 'w') as f:
+        with open('%s/run.sh' % hostdir, 'w', encoding='utf-8') as f:
             self._inject_testdef_parameters(f)
             f.write('set -e\n')
             f.write('export TESTRUN_ID=%s\n' % self.test_id)
@@ -613,6 +696,8 @@ class URLTestDefinition(object):
             steps = self.testdef['run'].get('steps', [])
             if steps:
                 for cmd in steps:
+                    if '--cmd' in cmd or '--shell' in cmd:
+                        cmd = re.sub(r'\$(\d+)\b', r'\\$\1', cmd)
                     f.write('%s\n' % cmd)
             f.write('echo "<LAVA_SIGNAL_ENDRUN $TESTRUN_ID $UUID>"\n')
             f.write('#wait for an ack from the dispatcher\n')
@@ -715,6 +800,8 @@ class cmd_lava_test_shell(BaseAction):
             'skip_install': {'type': 'string', 'optional': True},
             'lava_test_dir': {'type': 'string', 'optional': True},
             'lava_test_results_dir': {'type': 'string', 'optional': True},
+            'repeat': {'type': 'integer', 'optional': True},
+            'repeat_count': {'type': 'integer', 'optional': True},
         },
         'additionalProperties': False,
     }
@@ -727,12 +814,14 @@ class cmd_lava_test_shell(BaseAction):
         self._test_runs = []
         self._backup_bundle = {
             'test_runs': self._test_runs,
-            'format': 'Dashboard Bundle Format 1.7',
+            'format': 'Dashboard Bundle Format 1.7.1',
         }
 
     def run(self, testdef_urls=None, testdef_repos=None, timeout=-1, skip_install=None,
-            lava_test_dir=None, lava_test_results_dir=None):
+            lava_test_dir=None, lava_test_results_dir=None, repeat_count=0):
         target = self.client.target_device
+        global repeat_cnt
+        repeat_cnt = repeat_count
 
         delay = target.config.test_shell_serial_delay_ms
 
@@ -752,6 +841,9 @@ class cmd_lava_test_shell(BaseAction):
             if self.context.config.lava_proxy:
                 runner._connection.sendline(
                     "export http_proxy=%s" % self.context.config.lava_proxy, delay)
+            if self.context.config.lava_no_proxy:
+                runner._connection.sendline(
+                    "export no_proxy=%s" % self.context.config.lava_no_proxy, delay)
             runner._connection.sendline(
                 "%s/bin/lava-test-runner %s" % (
                     target.lava_test_dir,
@@ -878,7 +970,8 @@ class cmd_lava_test_shell(BaseAction):
         for fname in scripts_to_copy:
             with open(fname, 'r') as fin:
                 foutname = os.path.basename(fname)
-                with open('%s/bin/%s' % (mntdir, foutname), 'w') as fout:
+                with open('%s/bin/%s' % (mntdir, foutname), 'w',
+                          encoding='utf-8') as fout:
                     fout.write("#!%s\n\n" % shell)
                     fout.write(fin.read())
                     os.fchmod(fout.fileno(), XMOD)
@@ -892,7 +985,8 @@ class cmd_lava_test_shell(BaseAction):
         for fname in scripts_to_copy:
             with open(fname, 'r') as fin:
                 foutname = os.path.basename(fname)
-                with open('%s/bin/%s' % (mntdir, foutname), 'w') as fout:
+                with open('%s/bin/%s' % (mntdir, foutname), 'w',
+                          encoding='utf-8') as fout:
                     fout.write("#!%s\n\n" % shell)
                     # Target-specific scripts (add ENV to the generic ones)
                     if foutname == LAVA_GROUP_FILE:
@@ -919,6 +1013,9 @@ class cmd_lava_test_shell(BaseAction):
                     os.fchmod(fout.fileno(), XMOD)
 
     def _inject_lmp_api(self, mntdir, target):
+        """
+        This code is essentially unmaintained and may be dropped from the dispatcher in future.
+        """
         shell = target.deployment_data['lava_test_sh_cmd']
 
         # Generic scripts
@@ -927,7 +1024,8 @@ class cmd_lava_test_shell(BaseAction):
         for fname in scripts_to_copy:
             with open(fname, 'r') as fin:
                 foutname = os.path.basename(fname)
-                with open('%s/bin/%s' % (mntdir, foutname), 'w') as fout:
+                with open('%s/bin/%s' % (mntdir, foutname), 'w',
+                          encoding='utf-8') as fout:
                     fout.write("#!%s\n\n" % shell)
                     # Target-specific scripts (add ENV to the generic ones)
                     fout.write("LAVA_TEST_BIN='%s/bin'\n" %
@@ -977,7 +1075,8 @@ class cmd_lava_test_shell(BaseAction):
                 testdef.copy_test(hdir, tdir)
                 tdirs.append(tdir)
 
-            with open('%s/lava-test-runner.conf' % d, 'w') as f:
+            with open('%s/lava-test-runner.conf' % d, 'w',
+                      encoding='utf-8') as f:
                 for testdir in tdirs:
                     f.write('%s\n' % testdir)
 
@@ -999,7 +1098,7 @@ class cmd_lava_test_shell(BaseAction):
                 err_log = os.path.join(d, 'parse_err.log')
                 results_dir = os.path.join(d, 'results')
                 bundle = lava_test_shell.get_bundle(results_dir, testdef_objs, err_log)
-                parse_err_msg = read_content(err_log, ignore_missing=True)
+                parse_err_msg = utils.read_content(err_log, ignore_missing=True)
                 if os.path.isfile(err_log):
                     os.unlink(err_log)
                 # lava/results must be empty, but we keep a copy named
@@ -1037,7 +1136,7 @@ class cmd_lava_test_shell(BaseAction):
             raise GeneralError(parse_err_msg)
 
     def _handle_testrun(self, params):
-        test_id = params[0]
+        test_id = params[0].decode('utf-8')
         testdef = self._testdefs_by_name[test_id]
 
         now = datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%SZ')
