@@ -46,6 +46,7 @@ class DeviceSchema(schema.Schema):
     boot_cmds_android = schema.StringOption(fatal=True)  # And here
     boot_cmds_oe = schema.StringOption(fatal=True)  # And here?
     read_boot_cmds_from_image = schema.BoolOption(default=True)
+    boot_cmd_timeout = schema.IntOption(default=10)
     boot_options = schema.ListOption()
     boot_linaro_timeout = schema.IntOption(default=300)
     boot_part = schema.IntOption(fatal=True)
@@ -53,7 +54,11 @@ class DeviceSchema(schema.Schema):
     boot_retries = schema.IntOption(default=3)
     bootloader_prompt = schema.StringOption()
     bootloader_timeout = schema.IntOption(default=120)
+    image_boot_msg = schema.StringOption()
     image_boot_msg_timeout = schema.IntOption(default=120)
+    kernel_boot_msg = schema.StringOption()
+    kernel_boot_msg_timeout = schema.IntOption(default=120)
+    has_kernel_messages = schema.BoolOption(default=True)
     send_char = schema.BoolOption(default=True)
     test_image_prompts = schema.ListOption(default=["\(initramfs\)",
                                                     "linaro-test",
@@ -80,19 +85,15 @@ class DeviceSchema(schema.Schema):
     git_url_disablesuspend_sh = schema.StringOption()
     hard_reset_command = schema.StringOption()
     hostname = schema.StringOption()
-    image_boot_msg = schema.StringOption()
     interrupt_boot_command = schema.StringOption()
     interrupt_boot_prompt = schema.StringOption()
     interrupt_boot_control_character = schema.StringOption()
+    fvp_terminal_port_pattern = schema.StringOption(default="terminal_0: Listening for serial connection on port (\d+)")
     bootloader_serial_delay_ms = schema.IntOption(default=0)
     test_shell_serial_delay_ms = schema.IntOption(default=0)
     lmc_dev_arg = schema.StringOption()
     master_str = schema.StringOption(default="root@master")
     pre_connect_command = schema.StringOption()
-    qemu_binary = schema.StringOption(default="qemu-system-arm")
-    qemu_options = schema.StringOption()
-    qemu_drive_interface = schema.StringOption()
-    qemu_machine_type = schema.StringOption()
     power_on_cmd = schema.StringOption()  # for sdmux
     power_off_cmd = schema.StringOption()  # for sdmux
     reset_port_command = schema.StringOption()
@@ -184,6 +185,8 @@ class DeviceSchema(schema.Schema):
     partition_padding_string_org = schema.StringOption(default="p")
     partition_padding_string_android = schema.StringOption(default="p")
     android_serialno_patterns = schema.ListOption(default=['[0-9A-Fa-f]{16}'])
+    android_boot_uiautomator_jar = schema.StringOption(default=None)
+    android_boot_uiautomator_commands = schema.ListOption(default=None)
 
     arm_probe_binary = schema.StringOption(default='/usr/local/bin/arm-probe')
     arm_probe_config = schema.StringOption(default='/usr/local/etc/arm-probe-config')
@@ -240,6 +243,13 @@ class DeviceSchema(schema.Schema):
     dummy_ssh_username = schema.StringOption(default='root')
     dummy_ssh_identity_file = schema.StringOption(default=None)
 
+    # for qemu devices
+    qemu_binary = schema.StringOption(default="qemu-system-arm")
+    qemu_options = schema.StringOption()
+    qemu_drive_interface = schema.StringOption()
+    qemu_machine_type = schema.StringOption()
+    qemu_pflash = schema.ListOption(default=None)
+
     # for jtag devices
     jtag_driver = schema.StringOption(default=None)
     jtag_hard_reset_command = schema.StringOption(default=None)
@@ -257,8 +267,11 @@ class DeviceSchema(schema.Schema):
     adb_command = schema.StringOption()
     fastboot_command = schema.StringOption()
     fastboot_kernel_load_addr = schema.StringOption()
+    fastboot_efi_image = schema.StringOption(default=None)
+    run_boot_cmds = schema.BoolOption(default=False)
+    boot_fat_image_only = schema.BoolOption(default=False)
     rootfs_partition = schema.StringOption(default='userdata')
-    start_fastboot_command = schema.StringOption(default='fastboot')
+    start_fastboot_command = schema.StringOption(default=None)
     shared_working_directory = schema.StringOption(default=None)
 
     # for bootloader devices
@@ -269,8 +282,11 @@ class DeviceSchema(schema.Schema):
     u_load_addrs = schema.ListOption(default=None)
     z_load_addrs = schema.ListOption(default=None)
     uimage_only = schema.BoolOption(default=False)
+    text_offset = schema.StringOption(default=None)
+    multi_image_only = schema.BoolOption(default=False)
     uimage_xip = schema.BoolOption(default=False)
     append_dtb = schema.BoolOption(default=False)
+    prepend_blob = schema.StringOption(default=None)
 
     # for dynamic_vm devices
     dynamic_vm_backend_device_type = schema.StringOption(default='kvm')
@@ -317,6 +333,7 @@ class DispatcherSchema(schema.Schema):
     lava_image_tmpdir = schema.StringOption()
     lava_image_url = schema.StringOption()
     lava_proxy = schema.StringOption()
+    lava_no_proxy = schema.StringOption()
     lava_result_dir = schema.StringOption()
     lava_server_ip = schema.StringOption()
     lava_network_iface = schema.ListOption()
