@@ -29,7 +29,9 @@ from lava_dispatcher.client.base import (
     NetworkCommandRunner,
 )
 from lava_dispatcher.utils import (
-    extract_modules,
+    finalize_process,
+    connect_to_serial,
+    extract_overlay,
     extract_ramdisk,
     create_ramdisk,
     ensure_directory,
@@ -52,6 +54,7 @@ class BootloaderTarget(MasterImageTarget):
         super(BootloaderTarget, self).__init__(context, config)
         self._booted = False
         self._reset_boot = False
+        self._in_test_shell = False
         self._default_boot_cmds = 'boot_cmds_ramdisk'
         self._lava_nfsrootfs = None
         self._uboot_boot = False
@@ -65,16 +68,29 @@ class BootloaderTarget(MasterImageTarget):
         return prefix + '/' + self._get_rel_path(path, self._base_tmpdir)
 
     def _set_load_addresses(self, bootz):
+        meta = {}
         if not bootz and self.config.u_load_addrs and len(self.config.u_load_addrs) == 3:
             logging.info("Attempting to set uImage Load Addresses")
             self._boot_tags['{KERNEL_ADDR}'] = self.config.u_load_addrs[0]
             self._boot_tags['{RAMDISK_ADDR}'] = self.config.u_load_addrs[1]
             self._boot_tags['{DTB_ADDR}'] = self.config.u_load_addrs[2]
+            # Set boot metadata
+            meta['kernel-image'] = 'uImage'
+            meta['kernel-addr'] = self.config.u_load_addrs[0]
+            meta['initrd-addr'] = self.config.u_load_addrs[1]
+            meta['dtb-addr'] = self.config.u_load_addrs[2]
+            self.context.test_data.add_metadata(meta)
         elif bootz and self.config.z_load_addrs and len(self.config.z_load_addrs) == 3:
             logging.info("Attempting to set zImage Load Addresses")
             self._boot_tags['{KERNEL_ADDR}'] = self.config.z_load_addrs[0]
             self._boot_tags['{RAMDISK_ADDR}'] = self.config.z_load_addrs[1]
             self._boot_tags['{DTB_ADDR}'] = self.config.z_load_addrs[2]
+            # Set boot metadata
+            meta['kernel-image'] = 'zImage'
+            meta['kernel-addr'] = self.config.z_load_addrs[0]
+            meta['initrd-addr'] = self.config.z_load_addrs[1]
+            meta['dtb-addr'] = self.config.z_load_addrs[2]
+            self.context.test_data.add_metadata(meta)
         else:
             logging.debug("Undefined u_load_addrs or z_load_addrs. Three values required!")
 
@@ -139,9 +155,8 @@ class BootloaderTarget(MasterImageTarget):
         else:
             raise CriticalError("Unknown bootloader type")
 
-    def deploy_linaro_kernel(self, kernel, ramdisk, dtb, modules, rootfs,
-                             nfsrootfs, bootloader, firmware, bl1, bl2,
-                             bl31, rootfstype, bootloadertype, target_type):
+    def deploy_linaro_kernel(self, kernel, ramdisk, dtb, overlays, rootfs, nfsrootfs, bootloader, firmware, bl1, bl2,
+                             bl31, rootfstype, bootloadertype, target_type, qemu_pflash=None):
         if self.__deployment_data__ is None:
             # Get deployment data
             logging.debug("Attempting to set deployment data")
@@ -158,33 +173,39 @@ class BootloaderTarget(MasterImageTarget):
         # At a minimum we must have a kernel
         if kernel is None:
             raise CriticalError("No kernel image to boot")
-        if self._is_uboot() or self._is_uefi():
+        if self._is_uboot() or self._is_uefi() or self._is_ipxe():
             # Set the server IP (Dispatcher)
             self._boot_tags['{SERVER_IP}'] = self.context.config.lava_server_ip
             # We have been passed kernel image
             kernel = download_image(kernel, self.context,
                                     self._tmpdir, decompress=False)
-            if self.config.uimage_only and not is_uimage(kernel, self.context):
-                if len(self.config.u_load_addrs) == 3:
-                    kernel = create_uimage(kernel, self.config.u_load_addrs[0],
-                                           self._tmpdir, self.config.uimage_xip)
-                    logging.info('uImage created successfully')
-                else:
-                    logging.error('Undefined u_load_addrs, aborting uImage creation')
-
+            if self._is_uboot() or self._is_uefi():
+                if self.config.uimage_only and not is_uimage(kernel, self.context):
+                    if len(self.config.u_load_addrs) == 3:
+                        if self.config.text_offset:
+                            load_addr = self.config.text_offset
+                        else:
+                            load_addr = self.config.u_load_addrs[0]
+                        kernel = create_uimage(kernel, load_addr,
+                                               self._tmpdir, self.config.uimage_xip)
+                        logging.info('uImage created successfully')
+                    else:
+                        logging.error('Undefined u_load_addrs, aborting uImage creation')
             self._boot_tags['{KERNEL}'] = self._get_rel_path(kernel, self._base_tmpdir)
+
             if ramdisk is not None:
                 # We have been passed a ramdisk
                 ramdisk = download_image(ramdisk, self.context,
                                          self._tmpdir,
                                          decompress=False)
-                if modules is not None:
-                    modules = download_image(modules, self.context,
-                                             self._tmpdir,
-                                             decompress=False)
+                if overlays is not None:
                     ramdisk_dir = extract_ramdisk(ramdisk, self._tmpdir,
                                                   is_uboot=self._is_uboot_ramdisk(ramdisk))
-                    extract_modules(modules, ramdisk_dir)
+                    for overlay in overlays:
+                        overlay = download_image(overlay, self.context,
+                                                 self._tmpdir,
+                                                 decompress=False)
+                        extract_overlay(overlay, ramdisk_dir)
                     ramdisk = create_ramdisk(ramdisk_dir, self._tmpdir)
                 if self._is_uboot():
                     # Ensure ramdisk has u-boot header
@@ -222,11 +243,12 @@ class BootloaderTarget(MasterImageTarget):
                 self._lava_nfsrootfs = self._setup_nfs(nfsrootfs, self._tmpdir)
                 self._default_boot_cmds = 'boot_cmds_nfs'
                 self._boot_tags['{NFSROOTFS}'] = self._lava_nfsrootfs
-                if modules is not None and ramdisk is None:
-                    modules = download_image(modules, self.context,
-                                             self._tmpdir,
-                                             decompress=False)
-                    extract_modules(modules, self._lava_nfsrootfs)
+                if overlays is not None and ramdisk is None:
+                    for overlay in overlays:
+                        overlay = download_image(overlay, self.context,
+                                                 self._tmpdir,
+                                                 decompress=False)
+                        extract_overlay(overlay, self._lava_nfsrootfs)
             if bootloader is not None:
                 # We have been passed a bootloader
                 bootloader = download_image(bootloader, self.context,
@@ -245,41 +267,12 @@ class BootloaderTarget(MasterImageTarget):
                                                                           ramdisk,
                                                                           dtb)
 
-        elif self._is_ipxe():
-            # We have been passed kernel image
-            kernel = download_image(kernel, self.context,
-                                    self._tmpdir, decompress=False)
-            kernel_url = self._get_http_url(kernel)
-            self._boot_tags['{KERNEL}'] = kernel_url
-            # We have been passed a ramdisk
-            if ramdisk is not None:
-                # We have been passed a ramdisk
-                ramdisk = download_image(ramdisk, self.context,
-                                         self._tmpdir,
-                                         decompress=False)
-                if modules is not None:
-                    modules = download_image(modules, self.context,
-                                             self._tmpdir,
-                                             decompress=False)
-                    ramdisk_dir = extract_ramdisk(ramdisk, self._tmpdir,
-                                                  is_uboot=self._is_uboot_ramdisk(ramdisk))
-                    extract_modules(modules, ramdisk_dir)
-                    ramdisk = create_ramdisk(ramdisk_dir, self._tmpdir)
-                ramdisk_url = self._get_http_url(ramdisk)
-                self._boot_tags['{RAMDISK}'] = ramdisk_url
-            elif rootfs is not None:
-                # We have been passed a rootfs
-                rootfs = download_image(rootfs, self.context,
-                                        self._tmpdir, decompress=False)
-                rootfs_url = self._get_http_url(rootfs)
-                self._boot_tags['{ROOTFS}'] = rootfs_url
-
-    def deploy_linaro(self, hwpack, rfs, dtb, rootfstype, bootloadertype):
+    def deploy_linaro(self, hwpack, rfs, dtb, rootfstype, bootloadertype, qemu_pflash=None):
         self._uboot_boot = False
         super(BootloaderTarget, self).deploy_linaro(hwpack, rfs, dtb,
-                                                    rootfstype, bootloadertype)
+                                                    rootfstype, bootloadertype, qemu_pflash=qemu_pflash)
 
-    def deploy_linaro_prebuilt(self, image, dtb, rootfstype, bootloadertype):
+    def deploy_linaro_prebuilt(self, image, dtb, rootfstype, bootloadertype, qemu_pflash=None):
         self._uboot_boot = False
         if self._is_ipxe():
             if image is not None:
@@ -301,7 +294,8 @@ class BootloaderTarget(MasterImageTarget):
             super(BootloaderTarget, self).deploy_linaro_prebuilt(image,
                                                                  dtb,
                                                                  rootfstype,
-                                                                 bootloadertype)
+                                                                 bootloadertype,
+                                                                 qemu_pflash=qemu_pflash)
 
     def _run_boot(self):
         self._load_test_firmware()
@@ -313,14 +307,13 @@ class BootloaderTarget(MasterImageTarget):
             self.proc.sendline(self.config.pre_boot_cmd,
                                send_char=self.config.send_char)
         self._customize_bootloader(self.proc, boot_cmds)
-        self.proc.expect(self.config.image_boot_msg,
-                         timeout=self.config.image_boot_msg_timeout)
-        self._auto_login(self.proc)
-        self._wait_for_prompt(self.proc, self.config.test_image_prompts,
-                              self.config.boot_linaro_timeout)
+        self._monitor_boot(self.proc, self.tester_ps1, self.tester_ps1_pattern)
 
     def _boot_linaro_image(self):
-        self.proc.empty_buffer()
+        if self.proc:
+            finalize_process(self.proc)
+            self.proc = None
+        self.proc = connect_to_serial(self.context)
         if self._is_bootloader() and not self._booted:
             if self.config.hard_reset_command or self.config.hard_reset_command == "":
                 self._hard_reboot(self.proc)
@@ -328,14 +321,6 @@ class BootloaderTarget(MasterImageTarget):
             else:
                 self._soft_reboot(self.proc)
                 self._run_boot()
-            # When the kernel does DHCP which is the case for NFS/Ramdisk boot
-            # the nameserver data does get populated by the DHCP
-            # daemon. Thus, LAVA will populate the name server data.
-            self.proc.sendline('cat /proc/net/pnp > /etc/resolv.conf',
-                               send_char=self.config.send_char)
-            self.proc.sendline('export PS1="%s"'
-                               % self.tester_ps1,
-                               send_char=self.config.send_char)
             self._booted = True
         elif self._is_bootloader() and self._booted:
             self.proc.sendline('export PS1="%s"'
@@ -347,15 +332,18 @@ class BootloaderTarget(MasterImageTarget):
     def is_booted(self):
         return self._booted
 
-    def reset_boot(self):
+    def reset_boot(self, in_test_shell=True):
         self._reset_boot = True
+        self._booted = False
+        self._in_test_shell = in_test_shell
 
     @contextlib.contextmanager
     def file_system(self, partition, directory):
         if self._is_bootloader() and self._reset_boot:
-            self._booted = False
             self._reset_boot = False
-            raise Exception("Operation timed out, resetting platform!")
+            if self._in_test_shell:
+                self._in_test_shell = False
+                raise Exception("Operation timed out, resetting platform!")
         if self._is_bootloader() and not self._booted:
             self.context.client.boot_linaro_image()
         if self._is_bootloader() and self._lava_nfsrootfs:
