@@ -18,8 +18,6 @@
 # along
 # with this program; if not, see <http://www.gnu.org/licenses>.
 
-import copy
-import ConfigParser
 import contextlib
 import logging
 import os
@@ -44,6 +42,7 @@ class DynamicVmTarget(Target):
     supported_backends = {
         'kvm': lambda: kvm_adapter,
         'kvm-arm': lambda: kvm_adapter,
+        'kvm-aarch64': lambda: kvm_adapter,
     }
 
     def __init__(self, context, config):
@@ -69,17 +68,14 @@ class DynamicVmTarget(Target):
         self.backend_adapter.amend_config()
         return self.backend.power_on()
 
-    def deploy_linaro_kernel(self, kernel, ramdisk, dtb, modules, rootfs, nfsrootfs,
-                             bootloader, firmware, bl1, bl2, bl31, rootfstype,
-                             bootloadertype, target_type):
-        self.backend.deploy_linaro_kernel(kernel, ramdisk, dtb, modules, rootfs,
-                                          nfsrootfs, bootloader, firmware,
-                                          bl1, bl2, bl31, rootfstype,
-                                          bootloadertype, target_type)
+    def deploy_linaro_kernel(self, kernel, ramdisk, dtb, overlays, rootfs, nfsrootfs, bootloader, firmware, bl1, bl2,
+                             bl31, rootfstype, bootloadertype, target_type, qemu_pflash=None):
+        self.backend.deploy_linaro_kernel(kernel, ramdisk, dtb, overlays, rootfs, nfsrootfs, bootloader, firmware, bl1,
+                                          bl2, bl31, rootfstype, bootloadertype, target_type, qemu_pflash=qemu_pflash)
         self.backend_adapter.copy_images()
 
-    def deploy_linaro_prebuilt(self, image, dtb, rootfstype, bootloadertype):
-        self.backend.deploy_linaro_prebuilt(image, dtb, rootfstype, bootloadertype)
+    def deploy_linaro_prebuilt(self, image, dtb, rootfstype, bootloadertype, qemu_pflash=None):
+        self.backend.deploy_linaro_prebuilt(image, dtb, rootfstype, bootloadertype, qemu_pflash=qemu_pflash)
         self.backend_adapter.copy_images()
 
     def power_off(self, proc):
@@ -187,15 +183,19 @@ class kvm_adapter(object):
             run('fusermount -u %s' % dir_mount_point, failok=False)
 
     def cleanup(self):
-        image = self.device._sd_image
-        self.run_on_host(
-            'pkill -f "qemu-system-arm.*%s"' % image)
+        if self.device._sd_image:
+            image = self.device._sd_image
+        elif self.device._ramdisk:
+            image = self.device._ramdisk
+        if image:
+            self.run_on_host(
+                'pkill -f "qemu-system-arm.*%s"' % image, failok=True)
         self.run_on_host('rm -rf %s' % self.vm_images_dir)
 
-    def run_on_host(self, cmd):
+    def run_on_host(self, cmd, failok=False):
         ssh = self.ssh + ' -i %s' % self.identity_file
         run = self.device.context.run_command
-        run('%s root@%s -- %s' % (ssh, self.host, cmd), failok=False)
+        run('%s root@%s -- %s' % (ssh, self.host, cmd), failok)
 
     def scp_to_host(self, src, dst):
         scp = self.scp + ' -i %s' % self.identity_file
