@@ -45,6 +45,7 @@ class FastbootTarget(Target):
         self._target_type = None
         self._booted = False
         self._reset_boot = False
+        self._in_test_shell = False
         self._image_deployment = False
         self._ramdisk_deployment = False
         self._use_boot_cmds = False
@@ -58,9 +59,8 @@ class FastbootTarget(Target):
 
         self.driver = driver_class(self)
 
-    def deploy_linaro_kernel(self, kernel, ramdisk, dtb, modules, rootfs, nfsrootfs,
-                             bootloader, firmware, bl1, bl2, bl31, rootfstype,
-                             bootloadertype, target_type):
+    def deploy_linaro_kernel(self, kernel, ramdisk, dtb, overlays, rootfs, nfsrootfs, bootloader, firmware, bl1, bl2,
+                             bl31, rootfstype, bootloadertype, target_type, qemu_pflash=None):
         self._target_type = target_type
         self._use_boot_cmds = True
         if rootfs is not None:
@@ -72,12 +72,11 @@ class FastbootTarget(Target):
         attempts = 0
         deployed = False
         while (attempts < deploy_attempts) and (not deployed):
-            logging.info("Deploying test image. Attempt: %d" % (attempts + 1))
+            logging.info("Deploying test image. Attempt: %d", attempts + 1)
             try:
-                self._enter_fastboot()
-                self.driver.deploy_linaro_kernel(kernel, ramdisk, dtb, modules, rootfs, nfsrootfs,
-                                                 bootloader, firmware, bl1, bl2, bl31, rootfstype,
-                                                 bootloadertype, self._target_type, self.scratch_dir)
+                self.driver.deploy_linaro_kernel(kernel, ramdisk, dtb, overlays, rootfs, nfsrootfs, bootloader,
+                                                 firmware, bl1, bl2, bl31, rootfstype, bootloadertype,
+                                                 self._target_type, self.scratch_dir, qemu_pflash=qemu_pflash)
                 deployed = True
             except (subprocess.CalledProcessError, pexpect.TIMEOUT) as e:
                 msg = "Deployment failed: %s" % e
@@ -86,11 +85,12 @@ class FastbootTarget(Target):
                 continue
 
         if not deployed:
+            logging.error("Infrastructure Error: image deployment failed")
             msg = "Deployment Failed"
             logging.critical(msg)
             raise CriticalError(msg)
 
-    def deploy_android(self, boot, system, userdata, rootfstype,
+    def deploy_android(self, images, rootfstype,
                        bootloadertype, target_type):
         self._target_type = target_type
         self._image_deployment = True
@@ -99,10 +99,10 @@ class FastbootTarget(Target):
         attempts = 0
         deployed = False
         while (attempts < deploy_attempts) and (not deployed):
-            logging.info("Deploying test image. Attempt: %d" % (attempts + 1))
+            logging.info("Deploying test images image Attempt: %d", attempts + 1)
             try:
                 self._enter_fastboot()
-                self.driver.deploy_android(boot, system, userdata, rootfstype,
+                self.driver.deploy_android(images, rootfstype,
                                            bootloadertype, self._target_type,
                                            self.scratch_dir)
                 deployed = True
@@ -113,10 +113,18 @@ class FastbootTarget(Target):
                 continue
 
         if not deployed:
+            logging.error("Infrastructure Error: image deployment failed")
             msg = "Deployment Failed"
             logging.critical(msg)
             raise CriticalError(msg)
 
+    def dummy_deploy(self, target_type):
+        logging.info("Doing dummy deployment %s" % target_type)
+        self._image_deployment = True
+        self._target_type = target_type
+        self.deployment_data = deployment_data.get(self._target_type)
+        self.driver.dummy_deploy(target_type, self.scratch_dir)
+
     def get_device_version(self):
         # this is tricky, because fastboot does not have a visible version
         # number. For now let's use just the adb version number.
@@ -128,8 +136,9 @@ class FastbootTarget(Target):
     def is_booted(self):
         return self._booted
 
-    def reset_boot(self):
+    def reset_boot(self, in_test_shell=False):
         self._reset_boot = True
+        self._booted = False
 
     def power_on(self):
         try:
@@ -138,19 +147,25 @@ class FastbootTarget(Target):
                 return self.proc
             self._enter_fastboot()
             if self._use_boot_cmds:
-                boot_cmds = ''.join(self._load_boot_cmds(default=self.driver.get_default_boot_cmds()))
+                boot_cmds = self._load_boot_cmds(default=self.driver.get_default_boot_cmds(),
+                                                 boot_tags=self.driver.get_boot_tags())
                 self.driver.boot(boot_cmds)
             else:
                 self.driver.boot()
             if self.proc is None:
                 self.proc = self.driver.connect()
-            self._auto_login(self.proc)
-            self._wait_for_prompt(self.proc, self.config.test_image_prompts,
-                                  self.config.boot_linaro_timeout)
-            self._setup_prompt()
+            if self.config.run_boot_cmds:
+                self._enter_bootloader(self.proc)
+                boot_cmds = self._load_boot_cmds(default=self.driver.get_default_boot_cmds(),
+                                                 boot_tags=self.driver.get_boot_tags())
+                self._customize_bootloader(self.proc, boot_cmds)
+            self._monitor_boot(self.proc, self.tester_ps1, self.tester_ps1_pattern)
+            if self.config.start_fastboot_command:
+                self.driver.wait_for_adb()
             self._booted = True
             return self.proc
         except subprocess.CalledProcessError:
+            logging.info("Infrastructure Error: fastboot exception caught.")
             msg = 'Fastboot boot failed'
             raise OperationFailed(msg)
 
@@ -163,9 +178,10 @@ class FastbootTarget(Target):
     @contextlib.contextmanager
     def file_system(self, partition, directory):
         if self._reset_boot:
-            self._booted = False
             self._reset_boot = False
-            raise Exception("Operation timed out, resetting platform!")
+            if self._in_test_shell:
+                self._in_test_shell = False
+                raise Exception("Operation timed out, resetting platform!")
         if not self._booted:
             if self._target_type == 'android':
                 self.context.client.boot_linaro_android_image()
@@ -198,8 +214,7 @@ class FastbootTarget(Target):
             self.proc = None
         # Device needs to be forced into fastboot mode
         if not self.driver.in_fastboot():
-            if self.config.fastboot_driver == 'capri' or \
-               self.config.fastboot_driver == 'pxa1928dkb':
+            if self.config.start_fastboot_command:
                 # Connect to serial
                 self.proc = self.driver.connect()
                 # Hard reset the platform
