@@ -1,4 +1,5 @@
 # Copyright (C) 2012 Linaro Limited
+# Copyright (C) 2012 Linaro Limited
 #
 # Author: Andy Doan <andy.doan@linaro.org>
 #
@@ -48,12 +49,13 @@ from lava_dispatcher.test_data import (
 )
 from lava_dispatcher.errors import (
     CriticalError,
+    OperationFailed,
 )
 from lava_dispatcher.utils import (
     extract_tar,
     DrainConsoleOutput,
     finalize_process,
-    extract_modules,
+    extract_overlay,
     extract_ramdisk,
     create_ramdisk,
     ensure_directory,
@@ -64,7 +66,6 @@ from lava_dispatcher import deployment_data
 
 class FastModelTarget(Target):
 
-    PORT_PATTERN = 'terminal_0: Listening for serial connection on port (\d+)'
     ANDROID_WALLPAPER = 'system/wallpaper_info.xml'
     SYS_PARTITION = 2
     DATA_PARTITION = 5
@@ -86,6 +87,7 @@ class FastModelTarget(Target):
         self._ramdisk_boot = False
         self._booted = False
         self._reset_boot = False
+        self._in_test_shell = False
         self._bootloadertype = 'u_boot'
         self._boot_tags = {}
         self._scratch_dir = self.scratch_dir
@@ -218,15 +220,31 @@ class FastModelTarget(Target):
             logging.warning('No SECURE FLASHLOADER found, %r',
                             self.config.simulator_bl31_files)
 
-    def deploy_android(self, boot, system, data, rootfstype, bootloadertype,
+    def deploy_android(self, images, rootfstype, bootloadertype,
                        target_type):
         logging.info("Deploying Android on %s", self.config.hostname)
 
         self._bootloadertype = bootloadertype
+        self._boot = None
+        self._system = None
+        self._data = None
 
-        self._boot = download_image(boot, self.context, decompress=False)
-        self._data = download_image(data, self.context, decompress=False)
-        self._system = download_image(system, self.context, decompress=False)
+        for image in images:
+            if 'boot' in image['partition']:
+                self._boot = download_image(image['url'], self.context, decompress=False)
+            elif 'system' in image['partition']:
+                self._system = download_image(image['url'], self.context, decompress=False)
+            elif 'userdata' in image['partition']:
+                self._data = download_image(image['url'], self.context, decompress=False)
+            else:
+                msg = 'Unsupported partition option: %s' % image['partition']
+                logging.warning(msg)
+                raise CriticalError(msg)
+
+        if not all([self._boot, self._system, self._data]):
+            msg = 'Must supply a boot, system, and userdata image for fastmodel image deployment'
+            logging.warning(msg)
+            raise CriticalError(msg)
 
         self._sd_image = '%s/android.img' % os.path.dirname(self._system)
 
@@ -238,7 +256,7 @@ class FastModelTarget(Target):
 
         self._customize_android()
 
-    def deploy_linaro(self, hwpack, rootfs, dtb, rootfstype, bootloadertype):
+    def deploy_linaro(self, hwpack, rootfs, dtb, rootfstype, bootloadertype, qemu_pflash=None):
         hwpack = download_image(hwpack, self.context, decompress=False)
         rootfs = download_image(rootfs, self.context, decompress=False)
         odir = os.path.dirname(rootfs)
@@ -254,7 +272,7 @@ class FastModelTarget(Target):
         self._copy_needed_files_from_partition(self.config.root_part, 'boot')
         self._copy_needed_files_from_partition(self.config.root_part, 'lib')
 
-    def deploy_linaro_prebuilt(self, image, dtb, rootfstype, bootloadertype):
+    def deploy_linaro_prebuilt(self, image, dtb, rootfstype, bootloadertype, qemu_pflash=None):
         self._sd_image = download_image(image, self.context)
         self._bootloadertype = bootloadertype
         self.customize_image(self._sd_image)
@@ -266,9 +284,8 @@ class FastModelTarget(Target):
         self._copy_needed_files_from_partition(self.config.root_part, 'boot')
         self._copy_needed_files_from_partition(self.config.root_part, 'lib')
 
-    def deploy_linaro_kernel(self, kernel, ramdisk, dtb, modules, rootfs, nfsrootfs,
-                             bootloader, firmware, bl1, bl2, bl31, rootfstype,
-                             bootloadertype, target_type):
+    def deploy_linaro_kernel(self, kernel, ramdisk, dtb, overlays, rootfs, nfsrootfs, bootloader, firmware, bl1, bl2,
+                             bl31, rootfstype, bootloadertype, target_type, qemu_pflash=None):
         # Required
         if kernel is None:
             raise CriticalError("A kernel image is required")
@@ -278,7 +295,7 @@ class FastModelTarget(Target):
             raise CriticalError("A dtb is required")
 
         if rootfs is not None or nfsrootfs is not None or firmware is not None:
-            logging.warn("This platform only suports ramdisk booting, ignoring other parameters")
+            logging.warn("This platform only supports ramdisk booting, ignoring other parameters")
 
         self._ramdisk_boot = True
 
@@ -287,13 +304,14 @@ class FastModelTarget(Target):
         self._boot_tags['{KERNEL}'] = os.path.relpath(self._kernel, self._scratch_dir)
         self._initrd = download_image(ramdisk, self.context, self._scratch_dir,
                                       decompress=False)
-        if modules is not None:
-            modules = download_image(modules, self.context,
-                                     self._scratch_dir,
-                                     decompress=False)
+        if overlays is not None:
             ramdisk_dir = extract_ramdisk(self._initrd, self._scratch_dir,
                                           is_uboot=self._is_uboot_ramdisk(ramdisk))
-            extract_modules(modules, ramdisk_dir)
+            for overlay in overlays:
+                overlay = download_image(overlay, self.context,
+                                         self._scratch_dir,
+                                         decompress=False)
+                extract_overlay(overlay, ramdisk_dir)
             self._initrd = create_ramdisk(ramdisk_dir, self._scratch_dir)
         self._boot_tags['{RAMDISK}'] = os.path.relpath(self._initrd, self._scratch_dir)
         self._dtb = download_image(dtb, self.context, self._scratch_dir,
@@ -344,16 +362,19 @@ class FastModelTarget(Target):
     def is_booted(self):
         return self._booted
 
-    def reset_boot(self):
+    def reset_boot(self, in_test_shell=True):
         self._reset_boot = True
+        self._booted = False
+        self._in_test_shell = in_test_shell
 
     @contextlib.contextmanager
     def file_system(self, partition, directory):
         if self._ramdisk_boot:
             if self._reset_boot:
-                self._booted = False
                 self._reset_boot = False
-                raise Exception("Operation timed out, resetting platform!")
+                if self._in_test_shell:
+                    self._in_test_shell = False
+                    raise Exception("Operation timed out, resetting platform!")
             elif not self._booted:
                 self.context.client.boot_linaro_image()
             pat = self.tester_ps1_pattern
@@ -362,6 +383,7 @@ class FastModelTarget(Target):
             with self._busybox_file_system(runner, directory) as path:
                 yield path
         else:
+            self._check_power_state()
             with image_partition_mounted(self._sd_image, partition) as mntdir:
                 path = '%s/%s' % (mntdir, directory)
                 ensure_directory(path)
@@ -370,6 +392,7 @@ class FastModelTarget(Target):
     def extract_tarball(self, tarball_url, partition, directory='/'):
         logging.info('extracting %s to target', tarball_url)
 
+        self._check_power_state()
         with image_partition_mounted(self._sd_image, partition) as mntdir:
             tb = download_image(tarball_url, self.context, decompress=False)
             extract_tar(tb, '%s/%s' % (mntdir, directory))
@@ -388,8 +411,20 @@ class FastModelTarget(Target):
             for f in files:
                 os.chmod(os.path.join(root, f), 0o777)
 
+    def _check_power_state(self):
+        if self._sim_proc is not None:
+            logging.warning('device already powered on, powering off first')
+            self.power_off(None)
+
     def power_off(self, proc):
-        if self._uefi_vars is not None:
+        if self._sim_proc:
+            try:
+                self._soft_reboot(self.proc)
+            except KeyboardInterrupt:
+                raise KeyboardInterrupt
+            except:
+                logging.info('Graceful reboot of platform failed')
+        if self._uefi_vars is not None and self._sim_proc:
             logging.info('Requesting graceful shutdown')
             self._sim_proc.kill(signal.SIGTERM)
             self._sim_proc.wait()
@@ -412,18 +447,16 @@ class FastModelTarget(Target):
         DrainConsoleOutput(proc=self._sim_proc).start()
 
     def power_on(self):
-        if self.config.bridged_networking:
-            self._interface_name = os.path.basename(self._scratch_dir)
-            if not self._bridge_configured:
-                self._config_network_bridge(self.config.bridge_interface, self._interface_name)
         if self._ramdisk_boot and self._booted:
             self.proc.sendline('export PS1="%s"'
                                % self.tester_ps1,
                                send_char=self.config.send_char)
             return self.proc
-        if self._sim_proc is not None:
-            logging.warning('device already powered on, powering off first')
-            self.power_off(None)
+        self._check_power_state()
+        if self.config.bridged_networking:
+            self._interface_name = os.path.basename(self._scratch_dir)
+            if not self._bridge_configured:
+                self._config_network_bridge(self.config.bridge_interface, self._interface_name)
 
         self._check_needed_files()
 
@@ -450,7 +483,7 @@ class FastModelTarget(Target):
         # bl1 -> bl2 -> bl3.
         odir = os.path.dirname(self._sd_image)
         self._sim_proc = self.context.spawn(sim_cmd, cwd=odir, timeout=1200)
-        self._sim_proc.expect(self.PORT_PATTERN, timeout=300)
+        self._sim_proc.expect(self.config.fvp_terminal_port_pattern, timeout=300)
         self._serial_port = self._sim_proc.match.groups()[0]
         logging.info('serial console port on: %s', self._serial_port)
 
@@ -474,16 +507,10 @@ class FastModelTarget(Target):
                                              boot_tags=self._boot_tags)
             self._customize_bootloader(self.proc, boot_cmds)
 
+        self._monitor_boot(self.proc, self.tester_ps1, self.tester_ps1_pattern)
         self._auto_login(self.proc)
 
         if self._ramdisk_boot:
-            self._wait_for_prompt(self.proc, self.config.test_image_prompts,
-                                  self.config.boot_linaro_timeout)
-            self.proc.sendline('cat /proc/net/pnp > /etc/resolv.conf',
-                               send_char=self.config.send_char)
-            self.proc.sendline('export PS1="%s"'
-                               % self.tester_ps1,
-                               send_char=self.config.send_char)
             self._booted = True
 
         return self.proc
