@@ -26,21 +26,19 @@ import logging
 import time
 import pexpect
 import subprocess
+import lava_dispatcher.utils as utils
 
 from lava_dispatcher.device import boot_options
+from lava_dispatcher import deployment_data
 from lava_dispatcher.utils import (
     wait_for_prompt
 )
 from lava_dispatcher.client.lmc_utils import (
     image_partition_mounted
 )
-import lava_dispatcher.utils as utils
-from lava_dispatcher import deployment_data
-
 from lava_dispatcher.downloader import (
     download_image,
 )
-
 from lava_dispatcher.errors import (
     CriticalError,
     OperationFailed,
@@ -210,21 +208,23 @@ class Target(object):
         """
         raise NotImplementedError('power_on')
 
-    def deploy_linaro(self, hwpack, rfs, dtb, rootfstype, bootloadertype):
+    def deploy_linaro(self, hwpack, rfs, dtb, rootfstype, bootloadertype, qemu_pflash=None):
         raise NotImplementedError('deploy_image')
 
-    def deploy_android(self, boot, system, userdata, rootfstype,
+    def deploy_android(self, images, rootfstype,
                        bootloadertype, target_type):
         raise NotImplementedError('deploy_android_image')
 
-    def deploy_linaro_prebuilt(self, image, dtb, rootfstype, bootloadertype):
+    def deploy_linaro_prebuilt(self, image, dtb, rootfstype, bootloadertype, qemu_pflash=None):
         raise NotImplementedError('deploy_linaro_prebuilt')
 
-    def deploy_linaro_kernel(self, kernel, ramdisk, dtb, modules, rootfs,
-                             nfsrootfs, bootloader, firmware, bl1, bl2,
-                             bl31, rootfstype, bootloadertype, target_type):
+    def deploy_linaro_kernel(self, kernel, ramdisk, dtb, overlays, rootfs, nfsrootfs, bootloader, firmware, bl1, bl2,
+                             bl31, rootfstype, bootloadertype, target_type, qemu_pflash=None):
         raise NotImplementedError('deploy_linaro_kernel')
 
+    def dummy_deploy(self, target_type):
+        pass
+
     def power_off(self, proc):
         if proc is not None:
             proc.close()
@@ -235,7 +235,7 @@ class Target(object):
         # By default we pass
         pass
 
-    def reset_boot(self):
+    def reset_boot(self, in_test_shell=True):
         # By default we pass
         pass
 
@@ -331,11 +331,12 @@ class Target(object):
         if is_master:
             if self.config.master_login_prompt is not None:
                 self._wait_for_prompt(connection,
-                                      self.config.master_login_prompt, timeout=300)
+                                      self.config.master_login_prompt,
+                                      timeout=self.config.boot_linaro_timeout)
                 connection.sendline(self.config.master_username)
             if self.config.master_password_prompt is not None:
                 self._wait_for_prompt(connection,
-                                      self.config.master_password_prompt, timeout=300)
+                                      self.config.master_password_prompt, timeout=10)
                 connection.sendline(self.config.master_password)
             if self.config.master_login_commands is not None:
                 for command in self.config.master_login_commands:
@@ -343,11 +344,12 @@ class Target(object):
         else:
             if self.config.login_prompt is not None:
                 self._wait_for_prompt(connection,
-                                      self.config.login_prompt, timeout=300)
+                                      self.config.login_prompt,
+                                      timeout=self.config.boot_linaro_timeout)
                 connection.sendline(self.config.username)
             if self.config.password_prompt is not None:
                 self._wait_for_prompt(connection,
-                                      self.config.password_prompt, timeout=300)
+                                      self.config.password_prompt, timeout=10)
                 connection.sendline(self.config.password)
             if self.config.login_commands is not None:
                 for command in self.config.login_commands:
@@ -397,6 +399,7 @@ class Target(object):
         # Set flags
         boot_cmds_job_file = False
         boot_cmds_boot_options = False
+        master_boot = False
 
         # Set the default boot commands
         if default is None:
@@ -407,10 +410,12 @@ class Target(object):
         # Check for job defined boot commands
         if boot_cmds != 'boot_cmds_master':
             boot_cmds_job_file = self._is_job_defined_boot_cmds(self.config.boot_cmds)
+        else:
+            master_boot = True
 
         # Check if a user has entered boot_options
         options, user_option = boot_options.as_dict(self, defaults={'boot_cmds': boot_cmds})
-        if 'boot_cmds' in options and user_option:
+        if 'boot_cmds' in options and user_option and not master_boot:
             boot_cmds_boot_options = True
 
         # Interactive boot_cmds from the job file are a list.
@@ -467,16 +472,26 @@ class Target(object):
         else:
             connection.send("~$")
             connection.sendline("hardreset")
-            connection.empty_buffer()
 
     def _enter_bootloader(self, connection):
-        if connection.expect(self.config.interrupt_boot_prompt,
-                             timeout=self.config.bootloader_timeout) != 0:
-            raise Exception("Failed to enter bootloader")
-        if self.config.interrupt_boot_control_character:
-            connection.sendcontrol(self.config.interrupt_boot_control_character)
-        else:
-            connection.send(self.config.interrupt_boot_command)
+        try:
+            start = time.time()
+            connection.expect(self.config.interrupt_boot_prompt,
+                              timeout=self.config.bootloader_timeout)
+            if self.config.interrupt_boot_control_character:
+                connection.sendcontrol(self.config.interrupt_boot_control_character)
+            else:
+                connection.send(self.config.interrupt_boot_command)
+            # Record the time it takes to enter the bootloader.
+            enter_bootloader_time = "{0:.2f}".format(time.time() - start)
+            self.context.test_data.add_result('enter_bootloader', 'pass',
+                                              enter_bootloader_time, 'seconds')
+        except pexpect.TIMEOUT:
+            msg = 'Infrastructure Error: failed to enter the bootloader.'
+            logging.error(msg)
+            self.context.test_data.add_result('enter_bootloader', 'fail',
+                                              message=msg)
+            raise
 
     def _boot_cmds_preprocessing(self, boot_cmds):
         """ preprocess boot_cmds to prevent the boot procedure from some errors
@@ -501,42 +516,174 @@ class Target(object):
 
         return boot_cmds
 
+    def _monitor_boot(self, connection, ps1, ps1_pattern, is_master=False):
+
+        good = 'pass'
+        bad = 'fail'
+        if not is_master:
+            wait_for_image_boot = 'wait_for_image_boot_msg'
+            wait_for_kernel_boot = 'wait_for_kernel_boot_msg'
+            wait_for_login_prompt = 'wait_for_login_prompt'
+            kernel_exception = 'test_kernel_exception_'
+            wait_for_image_prompt = 'wait_for_test_image_prompt'
+            kernel_boot = 'test_kernel_boot_time'
+            userspace_boot = 'test_userspace_boot_time'
+        else:
+            wait_for_image_boot = 'wait_for_master_image_boot_msg'
+            wait_for_kernel_boot = 'wait_for_master_kernel_boot_msg'
+            wait_for_login_prompt = 'wait_for_master_login_prompt'
+            kernel_exception = 'master_kernel_exception_'
+            wait_for_image_prompt = 'wait_for_master_image_prompt'
+            kernel_boot = 'master_kernel_boot_time'
+            userspace_boot = 'master_userspace_boot_time'
+
+        if self.config.has_kernel_messages:
+            try:
+                connection.expect(self.config.image_boot_msg,
+                                  timeout=self.config.image_boot_msg_timeout)
+                start = time.time()
+                self.context.test_data.add_result(wait_for_image_boot,
+                                                  good)
+            except pexpect.TIMEOUT:
+                msg = "Kernel Error: did not start booting."
+                logging.error(msg)
+                self.context.test_data.add_result(wait_for_image_boot,
+                                                  bad, message=msg)
+                raise
+
+            try:
+                done = False
+                warnings = 0
+                while not done:
+                    pl = [self.config.kernel_boot_msg,
+                          'Freeing init memory',
+                          '-+\[ cut here \]-+\s+(.*\s+-+\[ end trace (\w*) \]-+)',
+                          '(Unhandled fault.*)\r\n']
+                    i = connection.expect(pl,
+                                          timeout=self.config.kernel_boot_msg_timeout)
+                    if i == 0 or i == 1:
+                        # Kernel booted normally
+                        done = True
+                    elif i == 2:
+                        warnings += 1
+                        logging.info('Kernel exception detected, logging error')
+                        kwarn = kernel_exception + str(warnings)
+                        kwarnings = connection.match.group(1)
+                        self.context.test_data.add_result(kwarn, 'fail', message=kwarnings)
+                        continue
+                    elif i == 3:
+                        warnings += 1
+                        logging.info('Kernel exception detected, logging error')
+                        kwarn = kernel_exception + str(warnings)
+                        kwarnings = connection.match.group(0)
+                        self.context.test_data.add_result(kwarn, 'fail', message=kwarnings)
+                        continue
+                kernel_boot_time = "{0:.2f}".format(time.time() - start)
+                self.context.test_data.add_result(wait_for_kernel_boot,
+                                                  good)
+                start = time.time()
+            except pexpect.TIMEOUT:
+                # Get the last line from the pexpect buffer
+                last_kmsg = connection.before.rstrip().split('\r\n')[-1]
+                msg = "Kernel Error:  %s " % last_kmsg
+                logging.error(msg)
+                kernel_boot_time = 0.0
+                self.context.test_data.add_result(wait_for_kernel_boot,
+                                                  bad, message=msg)
+                raise
+
+        try:
+            self._auto_login(connection, is_master)
+        except pexpect.TIMEOUT:
+            msg = "Userspace Error: auto login prompt not found."
+            logging.error(msg)
+            self.context.test_data.add_result(wait_for_login_prompt,
+                                              bad, message=msg)
+            raise
+
+        try:
+            if is_master:
+                pattern = self.config.master_str
+            else:
+                pattern = self.config.test_image_prompts
+
+            self._wait_for_prompt(connection, pattern, self.config.boot_linaro_timeout)
+            if not is_master:
+                if self.target_distro == 'android':
+                    # Gain root access
+                    connection.sendline('su')
+                    self._wait_for_prompt(connection, pattern, timeout=10)
+            connection.sendline('export PS1="%s"' % ps1,
+                                send_char=self.config.send_char)
+            self._wait_for_prompt(connection, ps1_pattern, timeout=10)
+            if self.config.has_kernel_messages:
+                userspace_boot_time = "{0:.2f}".format(time.time() - start)
+            self.context.test_data.add_result(wait_for_image_prompt, good)
+        except pexpect.TIMEOUT:
+            msg = "Userspace Error: image prompt not found."
+            logging.error(msg)
+            self.context.test_data.add_result(wait_for_image_prompt,
+                                              bad)
+            raise
+
+        # Record results
+        boot_meta = {}
+        boot_meta['dtb-append'] = str(self.config.append_dtb)
+        self.context.test_data.add_metadata(boot_meta)
+        if self.config.has_kernel_messages:
+            self.context.test_data.add_result(kernel_boot, 'pass',
+                                              kernel_boot_time, 'seconds')
+            self.context.test_data.add_result(userspace_boot, 'pass',
+                                              userspace_boot_time, 'seconds')
+            logging.info("Kernel boot time: %s seconds" % kernel_boot_time)
+            logging.info("Userspace boot time: %s seconds" % userspace_boot_time)
+
     def _customize_bootloader(self, connection, boot_cmds):
         start = time.time()
         delay = self.config.bootloader_serial_delay_ms
         _boot_cmds = self._boot_cmds_preprocessing(boot_cmds)
-        for line in _boot_cmds:
-            parts = re.match('^(?P<action>sendline|sendcontrol|expect)\s*(?P<command>.*)',
-                             line)
-            if parts:
-                try:
-                    action = parts.group('action')
-                    command = parts.group('command')
-                except AttributeError as e:
-                    raise Exception("Badly formatted command in \
-                                      boot_cmds %s" % e)
-                if action == "sendline":
-                    connection.send(command, delay,
-                                    send_char=self.config.send_char)
-                    connection.sendline('', delay,
-                                        send_char=self.config.send_char)
-                elif action == "sendcontrol":
-                    connection.sendcontrol(command)
-                elif action == "expect":
-                    command = re.escape(command)
-                    connection.expect(command, timeout=300)
-            else:
-                self._wait_for_prompt(connection,
-                                      self.config.bootloader_prompt,
-                                      timeout=300)
-                connection.sendline(line, delay,
-                                    send_char=self.config.send_char)
 
-        # Record boot time metadata
-        boottime = "{0:.2f}".format(time.time() - start)
-        boottime_meta = {'bootloader-load-time': boottime}
-        self.context.test_data.add_metadata(boottime_meta)
-        logging.debug("Bootloader load time: %s seconds", boottime)
+        try:
+            for line in _boot_cmds:
+                parts = re.match('^(?P<action>sendline|sendcontrol|expect)\s*(?P<command>.*)',
+                                 line)
+                if parts:
+                    try:
+                        action = parts.group('action')
+                        command = parts.group('command')
+                    except AttributeError as e:
+                        raise Exception("Badly formatted command in \
+                                          boot_cmds %s" % e)
+                    if action == "sendline":
+                        connection.send(command, delay,
+                                        send_char=self.config.send_char)
+                        connection.sendline('', delay,
+                                            send_char=self.config.send_char)
+                    elif action == "sendcontrol":
+                        connection.sendcontrol(command)
+                    elif action == "expect":
+                        command = re.escape(command)
+                        connection.expect(command,
+                                          timeout=self.config.boot_cmd_timeout)
+                else:
+                    self._wait_for_prompt(connection,
+                                          self.config.bootloader_prompt,
+                                          timeout=self.config.boot_cmd_timeout)
+                    connection.sendline(line, delay,
+                                        send_char=self.config.send_char)
+            self.context.test_data.add_result('execute_boot_cmds',
+                                              'pass')
+        except pexpect.TIMEOUT:
+            msg = "Bootloader Error: boot command execution failed."
+            logging.error(msg)
+            self.context.test_data.add_result('execute_boot_cmds',
+                                              'fail', message=msg)
+            raise
+
+        # Record boot_cmds execution time
+        execution_time = "{0:.2f}".format(time.time() - start)
+        self.context.test_data.add_result('boot_cmds_execution_time', 'pass',
+                                          execution_time, 'seconds')
 
     def _target_extract(self, runner, tar_file, dest, timeout=-1, busybox=False):
         tmpdir = self.context.config.lava_image_tmpdir
@@ -559,7 +706,7 @@ class Target(object):
             raise RuntimeError('bad file extension: %s' % tar_url)
 
         if busybox:
-            wget_options = ''
+            wget_options = '--proxy off'
         else:
             wget_options = '--no-check-certificate --no-proxy --connect-timeout=30 -S --progress=dot -e dotbytes=2M'
 
@@ -713,6 +860,10 @@ class Target(object):
         with open('%s/etc/hostname' % rootdir, 'w') as f:
             f.write('%s\n' % self.config.hostname)
 
+    @property
+    def target_distro(self):
+        return self.deployment_data['distro']
+
     @property
     def tester_ps1(self):
         return self._get_from_config_or_deployment_data('tester_ps1')
