@@ -30,6 +30,7 @@ import threading
 import time
 import urlparse
 import subprocess
+import re
 
 from shlex import shlex
 
@@ -62,9 +63,9 @@ def link_or_copy_file(src, dest):
         if err.errno == errno.EXDEV:
             shutil.copy(src, dest)
         if err.errno == errno.EEXIST:
-            logging.debug("Cached copy of %s already exists" % dest)
+            logging.debug("Cached copy of %s already exists", dest)
         else:
-            logging.exception("os.link '%s' with '%s' failed" % (src, dest))
+            logging.exception("os.link '%s' with '%s' failed", src, dest)
 
 
 def copy_file(src, dest):
@@ -147,14 +148,14 @@ def extract_rootfs(rootfs, root):
         raise CriticalError('Unable to remove tarball: %s' % rootfs)
 
 
-def extract_modules(modules, root):
-    """ Extracts the contents of a modules .tar.(bz2, gz, xz, lzma, etc) to the filesystem root.
+def extract_overlay(overlay, root):
+    """ Extracts the contents of a compressed tar (bz2, gz, xz, lzma, etc) to the filesystem root.
     """
-    logging.info('Attempting to install modules onto the filesystem')
-    if logging_system('nice tar --selinux -C %s -xaf %s' % (root, modules)):
-        raise CriticalError('Unable to extract tarball: %s to %s' % (modules, root))
-    if logging_system('rm %s' % modules):
-        raise CriticalError('Unable to remove tarball: %s' % modules)
+    logging.info('Attempting to install overlay onto the filesystem')
+    if logging_system('nice tar --selinux -C %s -xaf %s' % (root, overlay)):
+        raise CriticalError('Unable to extract tarball: %s to %s' % (overlay, root))
+    if logging_system('rm %s' % overlay):
+        raise CriticalError('Unable to remove tarball: %s' % overlay)
 
 
 def extract_ramdisk(ramdisk, tmpdir, is_uboot=False):
@@ -187,6 +188,8 @@ def create_ramdisk(ramdisk_dir, tmpdir):
         raise CriticalError('Unable to create cpio filesystem')
     if logging_system("cd %s && gzip %s" % (tmpdir, ramdisk_data)):
         raise CriticalError('Unable to compress cpio filesystem')
+    if logging_system("rm -rf %s" % ramdisk_dir):
+        raise CriticalError('Unable extracted ramdisk directory')
     return os.path.join(tmpdir, 'ramdisk.cpio.gz')
 
 
@@ -225,20 +228,83 @@ def create_uimage(kernel, load_addr, tmp_dir, xip, arch='arm'):
         raise CriticalError("uImage creation failed")
 
 
-def append_dtb(kernel, dtb, tmp_dir):
-    uimage_path = '%s/kernel-dtb' % tmp_dir
-    cmd = 'cat %s %s > %s' % (kernel, dtb, uimage_path)
+def create_multi_image(kernel, ramdisk, load_addr, tmp_dir, arch='arm'):
+    load_addr = int(load_addr, 16)
+    uimage_path = '%s/uImage' % tmp_dir
+    entry_addr = load_addr
+    cmd = 'mkimage -A %s -O linux -T multi \
+           -C none -a 0x%x -e 0x%x \
+            -d %s:%s %s' % (arch, load_addr,
+                            entry_addr, kernel,
+                            ramdisk, uimage_path)
 
-    logging.info('Appending dtb to kernel image')
+    logging.info('Creating Multi Image')
     logging.debug(cmd)
     r = subprocess.call(cmd, shell=True)
 
     if r == 0:
         return uimage_path
+    else:
+        raise CriticalError("Multi Image creation failed")
+
+
+def create_fat_boot_image(kernel, tmpdir, fastboot, dtb=None, ramdisk=None):
+    logging.info("Attempting to fat boot image")
+    boot_fat_dir = os.path.join(tmpdir, 'boot-fat')
+    boot_fat_img = os.path.join(tmpdir, 'boot-fat.img')
+    if logging_system("mkdir -p %s" % boot_fat_dir):
+        raise CriticalError("Unable to create boot-fat directory")
+    if logging_system("dd if=/dev/zero of=%s bs=512 count=131072" % boot_fat_img):
+        raise CriticalError("Unable to create boot-fat image")
+    if logging_system("mkfs.fat -n \"BOOT IMG\" %s" % boot_fat_img):
+        raise CriticalError("Unable to format boot-fat image")
+    if logging_system("mount -o loop,rw,sync %s %s" % (boot_fat_img, boot_fat_dir)):
+        raise CriticalError("Unable to mount boot-fat image")
+    if logging_system("cp -a %s %s" % (kernel, boot_fat_dir)):
+        raise CriticalError('Unable copy kernel')
+    if logging_system("cp -a %s %s/fastboot.efi" % (fastboot, boot_fat_dir)):
+        raise CriticalError('Unable copy fastboot image')
+    if ramdisk:
+        if logging_system("cp -a %s %s" % (ramdisk, boot_fat_dir)):
+            raise CriticalError('Unable copy ramdisk')
+    if dtb:
+        if logging_system("cp -a %s %s" % (dtb, boot_fat_dir)):
+            raise CriticalError('Unable copy dtb')
+    if logging_system("umount %s" % boot_fat_dir):
+        raise CriticalError("Unable to unmount boot-fat directory")
+    if logging_system("rm -rf %s" % boot_fat_dir):
+        raise CriticalError('Unable remove boot-fat directory')
+    return boot_fat_img
+
+
+def append_dtb(kernel, dtb, tmp_dir):
+    kernel_path = '%s/kernel-dtb' % tmp_dir
+    cmd = 'cat %s %s > %s' % (kernel, dtb, kernel_path)
+
+    logging.info('Appending dtb to kernel image')
+    logging.debug(cmd)
+    r = subprocess.call(cmd, shell=True)
+
+    if r == 0:
+        return kernel_path
     else:
         raise CriticalError("Appending dtb to kernel image failed")
 
 
+def prepend_blob(kernel, blob, tmp_dir):
+    kernel_path = '%s/kernel-blob' % tmp_dir
+    cmd = 'cat %s %s > %s' % (blob, kernel, kernel_path)
+
+    logging.info('Appending blob to kernel image')
+    logging.debug(cmd)
+    r = subprocess.call(cmd, shell=True)
+
+    if r == 0:
+        return kernel_path
+    else:
+        raise CriticalError("Appending blob to kernel image failed")
+
+
 def ensure_directory(path):
     """ ensures the path exists, if it doesn't it will be created
     """
@@ -272,7 +338,7 @@ def string_to_list(string):
 
 
 def logging_system(cmd):
-    logging.debug("Executing on host : '%r'" % cmd)
+    logging.debug("Executing on host : '%r'", cmd)
     return os.system(cmd)
 
 
@@ -291,10 +357,14 @@ class DrainConsoleOutput(threading.Thread):
             expect_end = time.time() + self.timeout
         while not self._stopevent.isSet():
             if expect_end and (expect_end <= time.time()):
-                logging.info("DrainConsoleOutput times out:%s" % self.timeout)
+                logging.info("DrainConsoleOutput times out:%s", self.timeout)
                 break
-            self.proc.empty_buffer()
-            time.sleep(5)
+            try:
+                self.proc.empty_buffer()
+                time.sleep(5)
+            except ValueError:
+                logging.debug("pexpect ended for thread %s", self.getName())
+                expect_end = time.time()
 
     def join(self, timeout=None):
         self._stopevent.set()
@@ -327,7 +397,7 @@ class logging_spawn(pexpect.spawn):
         return super(logging_spawn, self).sendcontrol(char)
 
     def send(self, string, delay=0, send_char=True):
-        logging.debug("send (delay_ms=%s): %s " % (delay, string))
+        logging.debug("send (delay_ms=%s): %s ", delay, string)
         sent = 0
         delay = float(delay) / 1000
         if send_char:
@@ -340,9 +410,10 @@ class logging_spawn(pexpect.spawn):
 
     def expect(self, *args, **kw):
         # some expect should not be logged because it is so much noise.
+        lava_logging = True
         if 'lava_no_logging' in kw:
             del kw['lava_no_logging']
-            return self.expect(*args, **kw)
+            lava_logging = False
 
         if 'timeout' in kw:
             timeout = kw['timeout']
@@ -350,9 +421,11 @@ class logging_spawn(pexpect.spawn):
             timeout = self.timeout
 
         if len(args) == 1:
-            logging.debug("expect (%d): '%s'", timeout, args[0])
+            if lava_logging:
+                logging.debug("expect (%d): '%s'", timeout, args[0])
         else:
-            logging.debug("expect (%d): '%s'", timeout, str(args))
+            if lava_logging:
+                logging.debug("expect (%d): '%s'", timeout, str(args))
 
         try:
             proc = super(logging_spawn, self).expect(*args, **kw)
@@ -366,7 +439,7 @@ class logging_spawn(pexpect.spawn):
         while index == 0:
             index = self.expect(
                 ['.+', pexpect.EOF, pexpect.TIMEOUT],
-                timeout=1, lava_no_logging=1)
+                timeout=0.1, lava_no_logging=1)
 
 
 def connect_to_serial(context):
@@ -392,28 +465,37 @@ def connect_to_serial(context):
         results.append(result)
 
     while retry_count < retry_limit:
-        proc = context.spawn(
-            context.device_config.connection_command,
-            timeout=1200)
-        logging.info('Attempting to connect to device using: %s' % context.device_config.connection_command)
-        match = proc.expect(patterns, timeout=10)
-        result = results[match]
-        logging.info('Matched %r which means %s', patterns[match], result)
-        if result == 'retry' or result == 'reset-port':
-            reset_cmd = context.device_config.reset_port_command
-            if reset_cmd:
-                logging.warning('attempting to reset serial port')
-                context.run_command(reset_cmd)
-            else:
-                logging.warning('no reset_port command configured')
-            proc.close(True)
+        try:
+            proc = context.spawn(
+                context.device_config.connection_command,
+                timeout=120)
+            logging.info('Attempting to connect to device using: %s', context.device_config.connection_command)
+            match = proc.expect(patterns, timeout=10)
+            result = results[match]
+            logging.info('Matched %r which means %s', patterns[match], result)
+            if result == 'retry' or result == 'reset-port':
+                reset_cmd = context.device_config.reset_port_command
+                if reset_cmd:
+                    logging.warning('attempting to reset serial port')
+                    context.run_command(reset_cmd)
+                else:
+                    logging.warning('no reset_port command configured')
+                proc.close(True)
+                retry_count += 1
+                time.sleep(5)
+                continue
+            elif result == 'all-good':
+                context.test_data.add_result('connect_to_console', 'pass')
+                atexit.register(proc.close, True)
+                return proc
+        except CriticalError:
             retry_count += 1
-            time.sleep(5)
-            continue
-        elif result == 'all-good':
-            atexit.register(proc.close, True)
-            return proc
-    raise CriticalError('could execute connection_command successfully')
+
+    msg = 'Infrastructure Error: cannot connect to console.'
+    logging.error(msg)
+    context.test_data.add_result('connect_to_console', 'fail',
+                                 message=msg)
+    raise CriticalError('could not execute connection_command successfully')
 
 
 def wait_for_prompt(connection, prompt_pattern, timeout):
@@ -453,10 +535,10 @@ def finalize_process(proc):
     if proc:
         try:
             os.killpg(proc.pid, signal.SIGKILL)
-            logging.debug("Finalizing child process group with PID %d" % proc.pid)
+            logging.debug("Finalizing child process group with PID %d", proc.pid)
         except OSError:
             proc.kill(9)
-            logging.debug("Finalizing child process with PID %d" % proc.pid)
+            logging.debug("Finalizing child process with PID %d", proc.pid)
         proc.close()
 
 
@@ -470,3 +552,22 @@ def read_content(filepath, ignore_missing=False):
 def write_content(filename, content):
     with open(filename, 'a') as f:
         f.write(content)
+
+
+def indices(string, char):
+    """
+    Return the indices of the given character in the given string.
+    Return an empty list if the character cannot be found.
+    """
+    return [i for i, c in enumerate(string) if c == char]
+
+
+def search_substr_from_array(string, array, sep=','):
+    """
+    Return True if any of the string in 'array' is a substring in 'string'
+    Strings in array are separated by default with ','
+    """
+    for obj in array.split(sep):
+        if re.search('.*' + str(obj) + '.*', str(string)):
+            return True
+    return False
